/*
 * starman: an iTunes rating interface
 * (c)2005 stephan.com
 * all rights reserved
 */
 
/*
 * we ALSO reload every time the user fires up dashboard
 */
var updateFrequency = 15*1000; // every 15 seconds (but only when Dashboard is visible)

var currentRating = 5;
var hands = new Array("h/h0.png","h/h1.png","h/h2.png","h/h3.png","h/h4.png","h/h5.png");

var debugEnabled = false;	
// TODO set this up later
var refreshInterval;

// Utility function for debugging.  When in Dashboard, alerts go to console.

function debug(text) {	 
    if (debugEnabled) {	 
        alert(text);	 
    }	 
}

// different for Dashboard vs. Safari
function gotoURL(url) {
	if (window.widget) {
		widget.openURL(url);
	} else {
		window.location.href = url;
	}
}

// Perform the flip from front to back.
function showPrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget) {
		window.resizeTo(172,132);
		widget.prepareForTransition("ToBack");		// freezes the widget so that you can change it without the user noticing
	} 
	
	front.style.display="none";		// hide the front
	back.style.display="block";		// show the back

	if (window.widget) {
		setTimeout ('widget.performTransition();', 0);	// and flip the widget over	
	}
}

// Perform the flip from back to front, refresh content

function hidePrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget) {
		widget.prepareForTransition("ToFront");		// freezes the widget and prepares it for the flip back to the front
	}
	
	back.style.display="none";			// hide the back
	front.style.display="block";		// show the front
	
	refreshTune();
	
	if (window.widget) {
		setTimeout ('widget.performTransition();', 0);		// and flip the widget back to the front
		window.resizeTo(123,176);
	}
}

// OK.  For some reason, if you throw an applescript with a conditional
// "tell" in it at osascript, it somehow pre-executes the tell that it
// isn't supposed to execute.  Therefore, it takes two calls to the
// system if you don't want iTunes to run on any refresh.  To further
// complicate matters, the call in the old version blocked any widget
// execution, which screwed up updating.  Grr.
//  The solution is now a three stage process using asynchronous system calls.
//
// Stage 1: Setup the check for iTunes.  When that completes it starts
// Stage 2: If iTunes isn't running, we're done.  Otherwise, fetch the current rating, which goes to
// Stage 3: Display the rating.
//
// This is, in effect, inside out and backwards from how it was done before.

// I'd like to recycle these and just use one, but I'm concerned about having two
// calls crank at the same time, might make a memory leak?  Also, I hate making these
// globals.  I know I could avoid that with an internal function, but fuck that.
var iTunesRunningCall = null;
var iTunesRatingCall = null;

function setRatingFromAS() {
	var tuneData = iTunesRatingCall.outputString;

	iTunesRatingCall = null; // throw out the old call

	if(!tuneData)
		currentRating = 0;
	else
		currentRating = parseInt(tuneData);
		
	debug("rating ="+currentRating);

	displayRating(currentRating);
}

function getRatingFromAS() {
	var iTunesStatus = iTunesRunningCall.outputString;
	
	debug("iTunesRunning? = "+iTunesStatus);

	if(iTunesStatus.indexOf("true")==-1) {
		debug("it not running");
		currentRating=0;
		displayRating(currentRating);
		return false;
	}
	
	iTunesRunningCall = null; // throw out the old call

	// iTunes is running.  Get the rating for the current song
	
	var iTunesApplescript="\
tell application \"iTunes\"\n\
	tell current track\n\
		((rating / 20) as integer) as string\n\
	end tell\n\
end tell";

debug(iTunesApplescript);

	var script = "/usr/bin/osascript -e '"+iTunesApplescript+"'";

	if(iTunesRatingCall != null) // if there's one of these running
		iTunesRatingCall.cancel(); // then kill it

	iTunesRatingCall = widget.system(script,setRatingFromAS);
}

function refreshTune() {
	//Thank god we only have to call this once.
	if(!window.widget) {
		debug("applescript not supported in Safari");
		currentStatus = "safari error";
		return;
	} else {
		debug("checking iTunes");
	}

	var iTunesApplescript = "\
tell application \"System Events\"\n\
	name of every process contains \"iTunes\"\n\
end tell";

debug(iTunesApplescript);
 
	var script = "/usr/bin/osascript -e '"+iTunesApplescript+"'";
	
	if(iTunesRunningCall != null) // if we already have one running
		iTunesRunningCall.cancel(); // then kill it.

	iTunesRunningCall = widget.system(script,getRatingFromAS);

}


// castrated version for demo
function setRating(r) {
	// display the OK sign to nag.
	document.getElementById("thehand").src = "h/ok.png";

	setTimeout('restoreRating();', 5000);
}

function displayRating(r) {
	document.getElementById("thehand").src = hands[r];
}

function restoreRating() {
	displayRating(currentRating);
}

// We don't want the widget refreshing when it's not on the screen, so these
// functions start and stop the refresh timer.

function setRefreshTimer() {	 
    if (refreshInterval === null) {	 
        debug("setting refresh timer");	 
        refreshInterval = setInterval("refreshTune();", updateFrequency);	 
    }	 
}	 
 	 
function clearRefreshTimer() {	 
    if (refreshInterval !== null) {	 
        debug("clearing refresh timer");	 
        clearInterval(refreshInterval);	 
        refreshInterval = null;
    }
}
               
function init() {

	if (window.widget) {	 
        widget.onshow = function OnShow() { // when Dashboard triggered	 
            debug("widget shown");	 
            setRefreshTimer();	 
            refreshTune();	 
        }	 
        widget.onremove = function OnRemove() { // when widget discarded 
            debug("widget removed");
            clearRefreshTimer();	 
        }	 
        widget.onhide = function OnHide() { // when Dashboard dismissed	 
            debug("widget hidden");	 
            clearRefreshTimer();	 
        }	 
    }
debug("init");    
 	refreshTune();

    setRefreshTimer();
}

// boilerplate code for infobutton animation taken from Apple

var flipShown = false;
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0,
				firstElement:null, timer:null};

function mousemove (event) {
	if (!flipShown) {
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer = null;
		}

		var starttime = (new Date).getTime() - 13;
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}

function mouseexit (event) {
	if (flipShown) {
		// fade in the info button
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer = null;
		}
		
		var starttime = (new Date).getTime() - 13;
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function animate() {
	var T;
	var ease;
	var time = (new Date).getTime();
	T = limit_3(time-animation.starttime, 0, animation.duration);
	if (T >= animation.duration) {
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	} else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}
	animation.firstElement.style.opacity = animation.now;
}

function limit_3 (a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
	return from + (to - from) * ease;
}
